"use strict";

var properties = {
  statusCode: {
    get: function get() {
      return this._response.statusCode;
    },
    set: function set(code) {
      this._explicitStatus = true;
      this._response.statusCode = code;
    }
  },

  end: {
    value: function value() {
      var _response;

      // Koa and Koa2 set the statusCode to `404` by default.
      // So we must do something as well as `ctx.body = body`.
      if (!this._explicitStatus) {
        // set to _response.statusCode
        this.statusCode = 200;
      }

      return (_response = this._response).end.apply(_response, arguments);
    }
  }
};

function makeResponse(res) {
  return Object.create({
    __proto__: res,
    _response: res
  }, properties);
}

function wrap(ctx, middleware, next) {
  return new Promise(function (resolve, reject) {
    middleware(ctx.req, makeResponse(ctx.res), function (err) {
      if (err) {
        reject(err);
        return;
      }

      resolve(next());
    });
  });
}

module.exports = function e2k(middleware) {
  return function (ctx, next) {
    return wrap(ctx, middleware, next);
  };
};